@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 
@@ -25,8 +25,8 @@
 #include "../gui/gui.h"
 
 extern void log_write_date (t_gui_buffer *);
+extern void log_write_line (t_gui_buffer *, char *);
 extern void log_write (t_gui_buffer *, char *);
-extern void log_write_line (t_gui_buffer *, t_gui_line *);
 extern void log_start (t_gui_buffer *);
 extern void log_end (t_gui_buffer *);
 
