@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* gui-display.c: display functions for Gtk GUI */
@@ -89,7 +89,7 @@ gui_assign_color (int *color, char *color_name)
     i = 0;
     while (gui_colors[i].name)
     {
-        if (strcasecmp (gui_colors[i].name, color_name) == 0)
+        if (ascii_strcasecmp (gui_colors[i].name, color_name) == 0)
         {
             *color = gui_colors[i].color;
             return 1;
@@ -114,7 +114,7 @@ gui_get_color_by_name (char *color_name)
     i = 0;
     while (gui_colors[i].name)
     {
-        if (strcasecmp (gui_colors[i].name, color_name) == 0)
+        if (ascii_strcasecmp (gui_colors[i].name, color_name) == 0)
             return gui_colors[i].color;
         i++;
     }
